/* suid root'able version of 'chroot'.  See 'info chroot'.
 * 
 * This binary has to be suid root as only root can use chroot() call.
 * 1. After that has been done, the real user ID and GID have to be
 *    restored so that user can't run anything with root priviledges.
 * 2. Then the program user gave, will be run as chrooted.
 * 
 * Copyright (c) 2002 Eero Tamminen <eero.tamminen@creanor.com>
 *
 * chroot-uid is licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <sys/resource.h>
#include "croco.h"

extern char **environ;

int main(int argc, char *argv[])
{
  char *path, *prog, *tmp_path, **args;
  const char *crocopath = CROCOPATH;
  const char *libcroco = LIBCROCO;
  int pathlen;
  
  /* check argument validity */
  if (argc < 3 || argv[1][0] != '/') {
    fprintf(stderr, "usage: %s   /absolute-path  path/program\n", *argv);
    fprintf(stderr, "  See 'man chroot'.  This is a version that can be run suid root\n");
    fprintf(stderr, "  as it restores real user ID before running the program.\n");
    fprintf(stderr, "  Path to the program should be relative to first path!\n");
    return -1;
  }
  path = argv[1];
  prog = argv[2];
  args = &(argv[2]);
  
  /* change to given directory and chroot to it */
  if (chdir(path) < 0) {
    perror("ERROR");
    fprintf(stderr, "chdir(%s)\n", path);
    return -1;
  }
  if (chroot(path) < 0) {
    perror("ERROR");
    fprintf(stderr, "chroot(%s)\n", path);
    return -1;
  }
  
  /* read _real_ user ID  and group and restore them */
  if (setuid(getuid()) < 0) {
    perror("ERROR");
    fprintf(stderr, "ERROR: setuid()\n");
    return -1;
  }
  if (setgid(getgid()) < 0) {
    perror("ERROR");
    fprintf(stderr, "ERROR: setgid()\n");
    return -1;
  }

  pathlen=strlen(crocopath)+strlen(libcroco)+1;
  tmp_path=malloc(pathlen);
  snprintf(tmp_path, pathlen, "%s%s", crocopath, libcroco);
  if (!access(tmp_path,R_OK)) {
    setenv("LD_PRELOAD", LIBCROCO, 1);
    setenv("LD_LIBRARY_PATH", CROCOPATH, 1);
  }
  /*
              UNIX is tough lover

     Apparently having ulimit -s unlimited kills ld.so
     So lock ulimit to it's current value.
  */

  {
    struct rlimit rlim;

    /* Set the stack limit huge so that alloca does not fail.  */
    if (getrlimit (RLIMIT_STACK, &rlim) == 0)
      {
        rlim.rlim_max=rlim.rlim_cur;
        setrlimit (RLIMIT_STACK, &rlim);
      }
  }

  /* run the given program */
  if (execve(prog, args, environ) < 0) {
    perror("ERROR");
    fprintf(stderr, "ERROR: execve(%s, %s..., %s...)\n", prog, *args, *environ);
    return -1;
  }
  return 0;
}
