/*
 * Copyright (C) 2006,2007 Lauri Leukkunen <lle@rahina.org>
 *
 * Licensed under LGPL version 2.1, see top level LGPL-2.1 file for details.
 */

#ifndef __CROCO_H
#define __CROCO_H

#include <stdio.h>
#include <time.h>
#include <stdarg.h>

#ifdef DMALLOC
#include <dmalloc.h>
#endif

#if defined(PATH_MAX)
#define CROCO_MAXPATH PATH_MAX
#elif defined(_POSIX_PATH_MAX)
#define CROCO_MAXPATH _POSIX_PATH_MAX
#elif defined(MAXPATHLEN)
#define CROCO_MAXPATH MAXPATHLEN
#else
#define CROCO_MAXPATH 2048
#endif

typedef int (*execve_fun)(const char *, char *const [], char *const []);
execve_fun next_execve;

/* extern int next_execve(const char *filename, char *const argv [],
			char *const envp[]); */

extern int do_exec(const char *exec_fn_name, const char *file,
		char *const *argv, char *const *envp);

/* ------ debug/trace logging system for sb2: */
#define SB_LOGLEVEL_uninitialized (-1)
#define SB_LOGLEVEL_NONE	0
#define SB_LOGLEVEL_ERROR	1
#define SB_LOGLEVEL_WARNING	2
#define SB_LOGLEVEL_NOTICE	3
#define SB_LOGLEVEL_INFO	5
#define SB_LOGLEVEL_DEBUG	8
#define SB_LOGLEVEL_NOISE	9
#define SB_LOGLEVEL_NOISE2	10
#define SB_LOGLEVEL_NOISE3	11

extern void sblog_init(void);
extern void sblog_vprintf_line_to_logfile(const char *file, int line,
	int level, const char *format, va_list ap);
extern void sblog_printf_line_to_logfile(const char *file, int line,
	int level, const char *format,...);

extern int sb_loglevel__; /* do not access directly */

int croco_map_path_for_exec(const char *my_file, char **mapping_result);

extern void croco_initialize_global_variables(void);

/* argvenvp */

int croco_execve_postprocess_native(
    char **mapped_file,
    char **filename,
    const char *binary_name,
    char ***argv,
    char ***envp);

int croco_execve_postprocess_cputransp(
    char **mapped_file,
    char **filename,
    const char *binary_name,
    char ***argv,
    char ***envp);

char *croco_execve_map_script_interpreter(
    const char *interpreter,
    const char *interp_arg,
    const char *mapped_script_filename,
    const char *orig_script_filename,
    char ***argv,
    char ***envp);



#define SB_LOG_INITIALIZED() (sb_loglevel__ >= SB_LOGLEVEL_NONE)

#define SB_LOG_IS_ACTIVE(level) ((level) <= sb_loglevel__)

#define SB_LOG(level, ...) \
	do { \
		if (SB_LOG_IS_ACTIVE(level)) { \
			sblog_printf_line_to_logfile( \
				__FILE__, __LINE__, level, __VA_ARGS__); \
		} \
	} while (0)

#define LIBCROCO    "libcroco.so"
#define CROCOPATH    "/croco/usr/lib/croco/"
#define LIBCPATH  "/croco/lib/libc.so.6"
#define LDPATH    "/croco/lib/ld-linux.so.2"
#define CPUTRAN    "/croco/usr/bin/qemu-arm"

#define HOST_LIB_PATH "/croco/lib/:/croco/usr/lib/:/croco/usr/lib/croco:/croco/usr/lib/libfakeroot"

extern int croco_global_vars_initialized__;
extern char *croco_orig_ld_preload;
extern char *croco_orig_ld_library_path;
extern char *croco_binary_name;
extern char *croco_real_binary_name;
extern char *croco_orig_binary_name;

#endif /* __CROCO_H */
