#!/bin/sh

usage()
{
	echo "usage: $0 /path/to/chroot [ -d cddir ] [ command ]"
	exit 1
}

if [ "x$1" = "x" ]; then usage; fi

CHROOT_BASE=`readlink -f $1`

shift

if [ "x$1" = "x" ]; then
	# no command given, login to shell
	RUN_COMMAND=shell
elif [ "x$1" = "x-d" ]; then
	if [ "x$2" = "x" ] || [ "x$3" = "x" ]; then usage; fi
	RUN_COMMAND=cd_run
	CD_DIR=$2;
	shift; shift;
else
	RUN_COMMAND=run
fi

if [ ! -d $CHROOT_BASE/croco ]; then
	EXEC_COMMAND=""
	TOOLS_BASE=""
else
	TOOLS_BASE=/croco
	# XXX: instead have a static binary inside chroot so that command passing works
	EXEC_COMMAND="/croco/lib/ld-linux.so.2 --library-path /croco/lib/:/croco/usr/lib/:/croco/usr/lib/croco"
	if [ ! -x $CHROOT_BASE/$TOOLS_BASE/lib/ld-linux.so.2 ]; then
		echo "$TOOLS_BASE exists but no lib/ld-linux.so.2 within"
		exit 1
	fi
fi

if [ -x $CHROOT_BASE/$TOOLS_BASE/bin/bash ]; then
	COMMAND=$TOOLS_BASE/bin/bash
elif [ -x $CHROOT_BASE/$TOOLS_BASE/bin/sh ]; then
	COMMAND=$TOOLS_BASE/bin/sh
else
	echo "ERROR: $TOOLS_BASE/bin/{bash,sh} not found under $CHROOT_BASE"
	exit 1
fi

if [ ! -x $CHROOT_BASE/$TOOLS_BASE/usr/bin/chroot-uid ]; then
	echo "ERROR: $CHROOT_BASE/$TOOLS_BASE/usr/bin/chroot-uid not found"
	exit 1
else
	if [ ! -u $CHROOT_BASE/$TOOLS_BASE/usr/bin/chroot-uid ] || [ ! -c $CHROOT_BASE/dev/null ]; then
		echo "ERROR: $CHROOT_BASE not setup by root"
		echo
		echo "to fix, run:"
		echo "sudo $CHROOT_BASE/$TOOLS_BASE/usr/lib/croco/setup-chroot.sh start $CHROOT_BASE"
		exit 1
	fi
	CHROOT_CMD=$CHROOT_BASE/$TOOLS_BASE/usr/bin/chroot-uid
fi

# setup user
USER_SETUP=$CHROOT_BASE/$TOOLS_BASE/usr/lib/croco/setup-chroot-user.sh
ROOT_SETUP=$CHROOT_BASE/$TOOLS_BASE/usr/lib/croco/setup-chroot.sh

if [ -x $USER_SETUP ]; then
	$USER_SETUP $CHROOT_BASE
fi
# go!
if [ "x$RUN_COMMAND" = "xshell" ]; then
	$CHROOT_CMD $CHROOT_BASE $EXEC_COMMAND $COMMAND
elif [ "x$RUN_COMMAND" = "xcd_run" ]; then
	echo "( cd $CD_DIR;$@ )"|$CHROOT_CMD $CHROOT_BASE $EXEC_COMMAND $COMMAND
else
	echo "$@"|$CHROOT_CMD $CHROOT_BASE $EXEC_COMMAND $COMMAND
fi

