/*
 * Copyright (C) 2003-2004 Nokia
 * Copyright (C) 2001      BitWagon Software LLC
 * Copyright (C) 1993-2003 Free Software Foundation
 *
 * This file may be used, modified, and distributed under the terms
 * of the GNU General Public License version 2.
 */

#define _GNU_SOURCE

#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/syscall.h>
#include <unistd.h>

#include <fcntl.h>
#include <stdarg.h>
#include <dlfcn.h>
#include <libgen.h>
#include <limits.h>
#include <errno.h>
#include <paths.h>
#include <sys/utsname.h>

#include "croco.h"

/*
 * execve wrapping is taken directly from fakeroot
 * please look at fakeroot source code for examples, documentation
 * and overall understanding how this works :-)
 * */

//#define DEBUG

#ifdef DEBUG
#define DEBUG_OUT(...) fprintf(stderr, __VA_ARGS__)
#else
#define DEBUG_OUT(...)
#endif


extern char **environ;


/*typedef int (*execve_fun)(const char *, char *const [], char *const []); */


#ifndef STATIC

#ifdef RTLD_NEXT
# define USE_RTLD_NEXT
#endif

struct next_wrap_st {
	void **doit;
	char *name;
};

static void *get_libc(void)
{
#ifndef USE_RTLD_NEXT
	static void *lib = 0;
	if (!lib) {
		lib = dlopen(LIBCPATH, RTLD_LAZY);
	}
	if (NULL == lib) {
		fprintf(stderr, "dlopen: %s: %s\n", LIBCPATH, dlerror());
		abort();
	}
	return lib;
#else
	return RTLD_NEXT;
#endif
}

static void load_library_symbols(void);
typedef int (*uname_fun)(struct utsname *buf);

extern execve_fun next_execve;
uname_fun next_uname;


static int tmp_execve(const char *file, char *const argv[], char *const envp[])
{
	load_library_symbols();
	return next_execve(file, argv, envp);
}

static int tmp_uname(struct utsname *buf)
{
	load_library_symbols();
	return next_uname(buf);
}

execve_fun next_execve=tmp_execve;
uname_fun next_uname=tmp_uname;

static struct next_wrap_st next_wrap[] = {
	{ (void (*)) &next_execve, "execve" },
	{ (void (*)) &next_uname,  "uname"  },
	{ NULL, NULL }
};

static void load_library_symbols(void)
{
	static int done = 0;
	int i;

	if (done) {
		return;
	}

	for (i = 0; next_wrap[i].doit; i++) {
		char *msg;
		*(next_wrap[i].doit) = dlsym(get_libc(), next_wrap[i].name);
		if ((msg = dlerror())) {
			fprintf(stderr, "dlsym(%s): %s\n",
				next_wrap[i].name, msg);
		}
	}
}

#else /* ifndef STATIC */


extern int __execve(const char *filename, char *const argv[], char *const envp[]);

execve_fun next_execve=__execve;
uname_fun next_uname=__uname;


#endif /* ifdef STATIC */

int
execve(const char *file,
	   char *const argv[],
	   char *const envp[])
{
	DEBUG_OUT("fake execve!\n");

	/* alien tools are executed explicitly with the dynamic linker */
	if (strcmp(file, LDPATH) == 0) {
/*		log_redirect(LOG_NONE, file, NULL, NULL, 0, 0);
		log_binary(BIN_SCRATCHBOX); */
#ifdef MEMCHECK
		exit(0);
#endif
		return next_execve(file, argv, envp);
	}

	return do_exec("execve", file, argv, envp);
}

/* ************ exec family of functions copied from glibc-2.3.2 ***********/

#ifndef	HAVE_GNU_LD
# define __environ	environ
#endif

/* Execute PATH with all arguments after PATH until
   a NULL pointer and environment from `environ'.  */
int
execl (const char *path, const char *arg, ...)
{
  size_t argv_max = 1024;
  const char **argv = alloca (argv_max * sizeof (const char *));
  unsigned int i;
  va_list args;

  argv[0] = arg;

  va_start (args, arg);
  i = 0;
  while (argv[i++] != NULL)
    {
      if (i == argv_max)
	{
	  const char **nptr = alloca ((argv_max *= 2) * sizeof (const char *));

#ifndef _STACK_GROWS_UP
	  if ((char *) nptr + argv_max == (char *) argv)
	    {
	      /* Stack grows down.  */
	      argv = (const char **) memcpy (nptr, argv,
					     i * sizeof (const char *));
	      argv_max += i;
	    }
	  else
#endif
#ifndef _STACK_GROWS_DOWN
	    if ((char *) argv + i == (char *) nptr)
	    /* Stack grows up.  */
	    argv_max += i;
	  else
#endif
	    /* We have a hole in the stack.  */
	    argv = (const char **) memcpy (nptr, argv,
					   i * sizeof (const char *));
	}

      argv[i] = va_arg (args, const char *);
    }
  va_end (args);

  return execve (path, (char *const *) argv, __environ);
}




/* Execute PATH with all arguments after PATH until a NULL pointer,
   and the argument after that for environment.  */
int
execle (const char *path, const char *arg, ...)
{
  size_t argv_max = 1024;
  const char **argv = alloca (argv_max * sizeof (const char *));
  const char *const *envp;
  unsigned int i;
  va_list args;
  argv[0] = arg;

  va_start (args, arg);
  i = 0;
  while (argv[i++] != NULL)
    {
      if (i == argv_max)
	{
	  const char **nptr = alloca ((argv_max *= 2) * sizeof (const char *));

#ifndef _STACK_GROWS_UP
	  if ((char *) nptr + argv_max == (char *) argv)
	    {
	      /* Stack grows down.  */
	      argv = (const char **) memcpy (nptr, argv,
					     i * sizeof (const char *));
	      argv_max += i;
	    }
	  else
#endif
#ifndef _STACK_GROWS_DOWN
	    if ((char *) argv + i == (char *) nptr)
	    /* Stack grows up.  */
	    argv_max += i;
	  else
#endif
	    /* We have a hole in the stack.  */
	    argv = (const char **) memcpy (nptr, argv,
					   i * sizeof (const char *));
	}

      argv[i] = va_arg (args, const char *);
    }

  envp = va_arg (args, const char *const *);
  va_end (args);

  return execve (path, (char *const *) argv, (char *const *) envp);
}




/* Execute FILE, searching in the `PATH' environment variable if
   it contains no slashes, with all arguments after FILE until a
   NULL pointer and environment from `environ'.  */
int
execlp (const char *file, const char *arg, ...)
{
  size_t argv_max = 1024;
  const char **argv = alloca (argv_max * sizeof (const char *));
  unsigned int i;
  va_list args;

  argv[0] = arg;

  va_start (args, arg);
  i = 0;
  while (argv[i++] != NULL)
    {
      if (i == argv_max)
	{
	  const char **nptr = alloca ((argv_max *= 2) * sizeof (const char *));

#ifndef _STACK_GROWS_UP
	  if ((char *) nptr + argv_max == (char *) argv)
	    {
	      /* Stack grows down.  */
	      argv = (const char **) memcpy (nptr, argv,
					     i * sizeof (const char *));
	      argv_max += i;
	    }
	  else
#endif
#ifndef _STACK_GROWS_DOWN
	    if ((char *) argv + i == (char *) nptr)
	    /* Stack grows up.  */
	    argv_max += i;
	  else
#endif
	    /* We have a hole in the stack.  */
	    argv = (const char **) memcpy (nptr, argv,
					   i * sizeof (const char *));
	}

      argv[i] = va_arg (args, const char *);
    }
  va_end (args);

  return execvp (file, (char *const *) argv);
}


#ifndef	HAVE_GNU_LD
# define __environ	environ
#endif

/* Execute PATH with arguments ARGV and environment from `environ'.  */
int
execv (const char *path, char *const argv[])
{
  return execve (path, argv, __environ);
}

/* Execute FILE, searching in the `PATH' environment variable if it contains
   no slashes, with arguments ARGV and environment from `environ'.  */
int
execvp (file, argv)
     const char *file;
     char *const argv[];
{
  if (*file == '\0')
    {
      /* We check the simple case first. */
      errno = ENOENT;
      return -1;
    }

  if (strchr (file, '/') != NULL)
    {
      /* Don't search when it contains a slash.  */
      execve (file, argv, __environ);

    }
  else
    {
      int got_eacces = 0;
      char *path, *p, *name;
      size_t len;
      size_t pathlen;

      path = getenv ("PATH");
      if (path == NULL)
	{
	  /* There is no `PATH' in the environment.
	     The default search path is the current directory
	     followed by the path `confstr' returns for `_CS_PATH'.  */
	  len = confstr (_CS_PATH, (char *) NULL, 0);
	  path = (char *) alloca (1 + len);
	  path[0] = ':';
	  (void) confstr (_CS_PATH, path + 1, len);
	}

      len = strlen (file) + 1;
      pathlen = strlen (path);
      name = alloca (pathlen + len + 1);
      /* Copy the file name at the top.  */
      name = (char *) memcpy (name + pathlen + 1, file, len);
      /* And add the slash.  */
      *--name = '/';

      p = path;
      do
	{
	  char *startp;

	  path = p;
	  p = strchrnul (path, ':');

	  if (p == path)
	    /* Two adjacent colons, or a colon at the beginning or the end
	       of `PATH' means to search the current directory.  */
	    startp = name + 1;
	  else
	    startp = (char *) memcpy (name - (p - path), path, p - path);

	  /* Try to execute this name.  If it works, execv will not return.  */
	  execve (startp, argv, __environ);

	  switch (errno)
	    {
	    case EACCES:
	      /* Record the we got a `Permission denied' error.  If we end
		 up finding no executable we can use, we want to diagnose
		 that we did find one but were denied access.  */
	      got_eacces = 1;
	    case ENOENT:
	    case ESTALE:
	    case ENOTDIR:
	      /* Those errors indicate the file is missing or not executable
		 by us, in which case we want to just try the next path
		 directory.  */
	      break;

	    default:
	      /* Some other error means we found an executable file, but
		 something went wrong executing it; return the error to our
		 caller.  */
	      return -1;
	    }
	}
      while (*p++ != '\0');

      /* We tried every element and none of them worked.  */
      if (got_eacces)
	/* At least one failure was due to permissions, so report that
           error.  */
	errno = EACCES;
    }

  /* Return the error from the last attempt (probably ENOENT).  */
  return -1;
}


#include	<sys/types.h>
#include	<sys/wait.h>
#include	<errno.h>
#include	<unistd.h>

int
system(const char *cmdstring)	/* version without signal handling */
{
	pid_t	pid;
	int		status;

	if (cmdstring == NULL)
		return(1);		/* always a command processor with Unix */

	if ( (pid = fork()) < 0) {
		status = -1;	/* probably out of processes */

	} else if (pid == 0) {				/* child */
		execl("/bin/sh", "sh", "-c", cmdstring, (char *) 0);
		_exit(127);		/* execl error */

	} else {							/* parent */
		while (waitpid(pid, &status, 0) < 0)
			if (errno != EINTR) {
				status = -1; /* error other than EINTR from waitpid() */
				break;
			}
	}

	return(status);
}

#include	<sys/wait.h>
#include	<errno.h>
#include	<fcntl.h>

static pid_t	*childpid = NULL;
						/* ptr to array allocated at run-time */
static int		maxfd;	/* from our open_max(), {Prog openmax} */

#define	SHELL	"/croco/tools/bin/bash"

FILE *
popen(const char *cmdstring, const char *type)
{
	int		i, pfd[2];
	pid_t	pid;
	FILE	*fp;

			/* only allow "r" or "w" */
	if ((type[0] != 'r' && type[0] != 'w') || type[1] != 0) {
		errno = EINVAL;		/* required by POSIX.2 */
		return(NULL);
	}

	if (childpid == NULL) {		/* first time through */
				/* allocate zeroed out array for child pids */
		maxfd = FOPEN_MAX;
		if ( (childpid = calloc(maxfd, sizeof(pid_t))) == NULL)
			return(NULL);
	}

	if (pipe(pfd) < 0)
		return(NULL);	/* errno set by pipe() */

	if ( (pid = fork()) < 0)
		return(NULL);	/* errno set by fork() */
	else if (pid == 0) {							/* child */
		if (*type == 'r') {
			close(pfd[0]);
			if (pfd[1] != STDOUT_FILENO) {
				dup2(pfd[1], STDOUT_FILENO);
				close(pfd[1]);
			}
		} else {
			close(pfd[1]);
			if (pfd[0] != STDIN_FILENO) {
				dup2(pfd[0], STDIN_FILENO);
				close(pfd[0]);
			}
		}
			/* close all descriptors in childpid[] */
		for (i = 0; i < maxfd; i++)
			if (childpid[i] > 0)
				close(i);

		execl(SHELL, "sh", "-c", cmdstring, (char *) 0);
		_exit(127);
	}
								/* parent */
	if (*type == 'r') {
		close(pfd[1]);
		if ( (fp = fdopen(pfd[0], type)) == NULL)
			return(NULL);
	} else {
		close(pfd[0]);
		if ( (fp = fdopen(pfd[1], type)) == NULL)
			return(NULL);
	}
	childpid[fileno(fp)] = pid;	/* remember child pid for this fd */
	return(fp);
}

int
pclose(FILE *fp)
{
	int		fd, stat;
	pid_t	pid;

	if (childpid == NULL)
		return(-1);		/* popen() has never been called */

	fd = fileno(fp);
	if ( (pid = childpid[fd]) == 0)
		return(-1);		/* fp wasn't opened by popen() */

	childpid[fd] = 0;
	if (fclose(fp) == EOF)
		return(-1);

	while (waitpid(pid, &stat, 0) < 0)
		if (errno != EINTR)
			return(-1);	/* error other than EINTR from waitpid() */

	return(stat);	/* return child's termination status */
}

static void set_name(char *str, const char *name)
{
	char *value = getenv(name);
	if (value) {
		strncpy(str, value, _UTSNAME_LENGTH);
	}
}

int uname(struct utsname *buf)
{
	if (next_uname(buf) < 0) {
		return -1;
	}

	set_name(buf->sysname,  "CROCO_UNAME_SYSNAME");
	set_name(buf->nodename, "CROCO_UNAME_NODENAME");
	set_name(buf->release,  "CROCO_UNAME_RELEASE");
	set_name(buf->version,  "CROCO_UNAME_VERSION");
	set_name(buf->machine,  "CROCO_UNAME_MACHINE");

	return 0;
}

