#!/bin/sh

COMMAND=$1

usage() {
	echo usage:
	echo "$0 [ start | stop ] /path/to/chroot "
}

extract_devices()
{
	DEVTAR=usr/share/debootstrap/devices.tar.gz
	if [ -r $1/$DEVTAR ]; then 
		FULL_DEVTAR=$1/$DEVTAR;
	elif [ -r $1/croco/$DEVTAR  ]; then
		FULL_DEVTAR=$1/croco/$DEVTAR;
	elif [ -r /$DEVTAR ]; then
		FULL_DEVTAR=/$DEVTAR;
	else
		echo ERROR: devices.tar.gz could not be located, please install debootstrap
		exit 1;
	fi
	(
		cd $1
		tar xzf $FULL_DEVTAR
	)
}

start_chroot() {
	CHROOT_BASE=$1
	if [ ! -c $CHROOT_BASE/dev/null ]; then
		extract_devices $CHROOT_BASE
	fi
	passwd_owner=`stat -c %U $CHROOT_BASE/etc/passwd`
	# if chroot is extracted as r00t, then we need to edit passwd as root
	if [ x$passwd_owner = xroot ] && [ x$SUDO_USER != x ]; then
		 `readlink -f $(dirname $0)`/setup-chroot-user.sh $CHROOT_BASE $SUDO_USER
	fi
	# XXX read a config file within $CHROOT_BASE for mounts 
	# XXX bindmount home,tmp
	# The key problem is security, should we just ignore it?

	if [ ! -L $CHROOT_BASE/proc/self/exe ]; then
		mount -t proc "proc-chroot" $CHROOT_BASE/proc
	fi
	if [ ! -c $CHROOT_BASE/dev/pts/0 ]; then
		mkdir -p $CHROOT_BASE/dev/pts
		mount -t devpts "devtps-chroot" $CHROOT_BASE/dev/pts
	fi
}

stop_chroot() {
	CHROOT_BASE=$1
	# XXX kill all processess within

	# simply umount all under the chroot_base
	mount_list=`grep $1 /proc/mounts|sed -e 's,.* /,/,' -e 's, .*,,'`
	for mount in $mount_list; do
		echo unmounting "$mount"
		umount -l $mount
	done

}

sanity_check() {
	CHROOT_BASE=$1

	if [ ! -d $CHROOT_BASE/croco ]; then
		echo "INFO: /croco subdir missing, assuming setting up tools chroot"
		TOOLS_BASE=""
	else
		TOOLS_BASE="croco"
	fi

	QEMU=$CHROOT_BASE/$TOOLS_BASE/usr/bin/qemu-arm
	if [ ! -x $QEMU ]; then
		echo "WARNING: $QEMU missing, arm binaries won't work"
	fi

	LIBCROCO=$CHROOT_BASE/$TOOLS_BASE/usr/lib/croco/libcroco.so
	if [ ! -r $LIBCROCO ]; then
		echo "WARNING: $LIBCROCO missing"
	fi

	CHROOT_UID=$CHROOT_BASE/$TOOLS_BASE/usr/bin/chroot-uid
	if [ ! -x $CHROOT_UID ]; then
		echo "ERROR: $CHROOT_UID not found"
		exit 1
	else
		# XXX: A security hole size of texas
		chown root:root $CHROOT_UID
		chmod a+s $CHROOT_UID
	fi
}

if [ x$2 = x ]
then
	usage;
	exit 1
fi

if [ x`whoami` != xroot ]
then
	echo "$0 needs to be run as root"
	exit 1
fi

if [ x$COMMAND = xstart ]
then
	sanity_check $2
	start_chroot $2
elif [ x$COMMAND = xstop ]
then
	stop_chroot $2
else
	usage;
	exit 1
fi


